@@ -1,26 +1,25 @@
-from argparse import ArgumentParser
+from argparse import ArgumentParser, ArgumentTypeError
 from collections import OrderedDict
 from copy import deepcopy
 from functools import partial
 import json
+from pathlib import Path
 from pprint import pformat
-from shlex import quote
+from shlex import quote, split
 import sys
 import time
 from types import MethodType
 from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, TypeVar, Union, get_type_hints
-from typing_inspect import is_literal_type, get_args
+from typing_inspect import is_literal_type
 from warnings import warn
 
 from tap.utils import (
     get_class_variables,
+    get_args,
     get_argument_name,
-    get_git_root,
     get_dest,
-    get_git_url,
     get_origin,
-    has_git,
-    has_uncommitted_changes,
+    GitInfo,
     is_option_arg,
     type_to_str,
     get_literals,
@@ -32,11 +31,15 @@ from tap.utils import (
     enforce_reproducibility,
 )
 
+if sys.version_info >= (3, 10):
+    from types import UnionType
+
 
 # Constants
 EMPTY_TYPE = get_args(List)[0] if len(get_args(List)) > 0 else tuple()
 BOXED_COLLECTION_TYPES = {List, list, Set, set, Tuple, tuple}
-OPTIONAL_TYPES = {Optional, Union}
+UNION_TYPES = {Union} | ({UnionType} if sys.version_info >= (3, 10) else set())
+OPTIONAL_TYPES = {Optional} | UNION_TYPES
 BOXED_TYPES = BOXED_COLLECTION_TYPES | OPTIONAL_TYPES
 
 
@@ -171,13 +174,30 @@ class Tap(ArgumentParser):
 
             # If type is not explicitly provided, set it if it's one of our supported default types
             if 'type' not in kwargs:
-
-                # Unbox Optional[type] and set var_type = type
+                # Unbox Union[type] (Optional[type]) and set var_type = type
                 if get_origin(var_type) in OPTIONAL_TYPES:
                     var_args = get_args(var_type)
 
+                    # If type is Union or Optional without inner types, set type to equivalent of Optional[str]
+                    if len(var_args) == 0:
+                        var_args = (str, type(None))
+
+                    # Raise error if type function is not explicitly provided for Union types (not including Optionals)
+                    if get_origin(var_type) in UNION_TYPES and not (len(var_args) == 2 and var_args[1] == type(None)):
+                        raise ArgumentTypeError(
+                            'For Union types, you must include an explicit type function in the configure method. '
+                            'For example,\n\n'
+                            'def to_number(string: str) -> Union[float, int]:\n'
+                            '    return float(string) if \'.\' in string else int(string)\n\n'
+                            'class Args(Tap):\n'
+                            '    arg: Union[float, int]\n'
+                            '\n'
+                            '    def configure(self) -> None:\n'
+                            '        self.add_argument(\'--arg\', type=to_number)'
+                        )
+
                     if len(var_args) > 0:
-                        var_type = get_args(var_type)[0]
+                        var_type = var_args[0]
 
                         # If var_type is tuple as in Python 3.6, change to a typing type
                         # (e.g., (typing.List, <class 'bool'>) ==> typing.List[bool])
@@ -189,12 +209,14 @@ class Tap(ArgumentParser):
                 # First check whether it is a literal type or a boxed literal type
                 if is_literal_type(var_type):
                     var_type, kwargs['choices'] = get_literals(var_type, variable)
+
                 elif (get_origin(var_type) in (List, list, Set, set)
                       and len(get_args(var_type)) > 0
                       and is_literal_type(get_args(var_type)[0])):
                     var_type, kwargs['choices'] = get_literals(get_args(var_type)[0], variable)
                     if kwargs.get('action') not in {'append', 'append_const'}:
                         kwargs['nargs'] = kwargs.get('nargs', '*')
+
                 # Handle Tuple type (with type args) by extracting types of Tuple elements and enforcing them
                 elif get_origin(var_type) in (Tuple, tuple) and len(get_args(var_type)) > 0:
                     loop = False
@@ -202,8 +224,8 @@ class Tap(ArgumentParser):
 
                     # Don't allow Tuple[()]
                     if len(types) == 1 and types[0] == tuple():
-                        raise ValueError('Empty Tuples (i.e. Tuple[()]) are not a valid Tap type '
-                                         'because they have no arguments.')
+                        raise ArgumentTypeError('Empty Tuples (i.e. Tuple[()]) are not a valid Tap type '
+                                                'because they have no arguments.')
 
                     # Handle Tuple[type, ...]
                     if len(types) == 2 and types[1] == Ellipsis:
@@ -239,7 +261,7 @@ class Tap(ArgumentParser):
                         kwargs['type'] = boolean_type
                         kwargs['choices'] = [True, False]  # this makes the help message more helpful
                     else:
-                        action_cond = "true" if kwargs.get("required", False) or not kwargs["default"] else "false"
+                        action_cond = 'true' if kwargs.get('required', False) or not kwargs['default'] else 'false'
                         kwargs['action'] = kwargs.get('action', f'store_{action_cond}')
                 elif kwargs.get('action') not in {'count', 'append_const'}:
                     kwargs['type'] = var_type
@@ -337,7 +359,7 @@ class Tap(ArgumentParser):
         pass
 
     @staticmethod
-    def get_reproducibility_info() -> Dict[str, str]:
+    def get_reproducibility_info(repo_path: Optional[str] = None) -> Dict[str, str]:
         """Gets a dictionary of reproducibility information.
 
         Reproducibility information always includes:
@@ -350,38 +372,50 @@ class Tap(ArgumentParser):
                    Ex. https://github.com/swansonk14/rationale-alignment/tree/<hash>
         - git_has_uncommitted_changes: Whether the current git repo has uncommitted changes.
 
+        :param repo_path: Path to the git repo to examine for reproducibility info.
+                          If None, uses the git repo of the Python file that is run.
         :return: A dictionary of reproducibility information.
         """
+        # Get the path to the Python file that is being run
+        if repo_path is None:
+            repo_path = (Path.cwd() / Path(sys.argv[0]).parent).resolve()
+
         reproducibility = {
             'command_line': f'python {" ".join(quote(arg) for arg in sys.argv)}',
             'time': time.strftime('%c')
         }
 
-        if has_git():
-            reproducibility['git_root'] = get_git_root()
-            reproducibility['git_url'] = get_git_url(commit_hash=True)
-            reproducibility['git_has_uncommitted_changes'] = has_uncommitted_changes()
+        git_info = GitInfo(repo_path=repo_path)
+
+        if git_info.has_git():
+            reproducibility['git_root'] = git_info.get_git_root()
+            reproducibility['git_url'] = git_info.get_git_url(commit_hash=True)
+            reproducibility['git_has_uncommitted_changes'] = git_info.has_uncommitted_changes()
 
         return reproducibility
 
-    def _log_all(self) -> Dict[str, Any]:
+    def _log_all(self, repo_path: Optional[str] = None) -> Dict[str, Any]:
         """Gets all arguments along with reproducibility information.
 
+        :param repo_path: Path to the git repo to examine for reproducibility info.
+                          If None, uses the git repo of the Python file that is run.
         :return: A dictionary containing all arguments along with reproducibility information.
         """
         arg_log = self.as_dict()
-        arg_log['reproducibility'] = self.get_reproducibility_info()
+        arg_log['reproducibility'] = self.get_reproducibility_info(repo_path=repo_path)
 
         return arg_log
 
     def parse_args(self: TapType,
                    args: Optional[Sequence[str]] = None,
-                   known_only: bool = False) -> TapType:
+                   known_only: bool = False,
+                   legacy_config_parsing = False) -> TapType:
         """Parses arguments, sets attributes of self equal to the parsed arguments, and processes arguments.
 
         :param args: List of strings to parse. The default is taken from `sys.argv`.
         :param known_only: If true, ignores extra arguments and only parses known arguments.
         Unparsed arguments are saved to self.extra_args.
+        :legacy_config_parsing: If true, config files are parsed using `str.split` instead of `shlex.split`.
         :return: self, which is a Tap instance containing all of the parsed args.
         """
         # Prevent double parsing
@@ -389,7 +423,13 @@ class Tap(ArgumentParser):
             raise ValueError('parse_args can only be called once.')
 
         # Collect arguments from all of the configs
-        config_args = [arg for args_from_config in self.args_from_configs for arg in args_from_config.split()]
+
+        if legacy_config_parsing:
+            splitter = lambda arg_string: arg_string.split()
+        else:
+            splitter = lambda arg_string: split(arg_string, comments=True)
+
+        config_args = [arg for args_from_config in self.args_from_configs for arg in splitter(args_from_config)]
 
         # Add config args at lower precedence and extract args from the command line if they are not passed explicitly
         args = config_args + (sys.argv[1:] if args is None else list(args))
@@ -590,12 +630,17 @@ class Tap(ArgumentParser):
 
         return self
 
-    def save(self, path: str, with_reproducibility: bool = True, skip_unpicklable: bool = False) -> None:
+    def save(self,
+             path: str, with_reproducibility: bool = True,
+             skip_unpicklable: bool = False,
+             repo_path: Optional[str] = None) -> None:
         """Saves the arguments and reproducibility information in JSON format, pickling what can't be encoded.
 
         :param path: Path to the JSON file where the arguments will be saved.
         :param with_reproducibility: If True, adds a "reproducibility" field with information (e.g. git hash)
                                      to the JSON file.
+        :param repo_path: Path to the git repo to examine for reproducibility info.
+                          If None, uses the git repo of the Python file that is run.
         :param skip_unpicklable: If True, does not save attributes whose values cannot be pickled.
         """
         with open(path, 'w') as f:
@@ -605,7 +650,8 @@ class Tap(ArgumentParser):
     def load(self,
              path: str,
              check_reproducibility: bool = False,
-             skip_unsettable: bool = False) -> TapType:
+             skip_unsettable: bool = False,
+             repo_path: Optional[str] = None) -> TapType:
         """Loads the arguments in JSON format. Note: Due to JSON, tuples are loaded as lists.
 
         :param path: Path to the JSON file where the arguments will be loaded from.
@@ -613,6 +659,8 @@ class Tap(ArgumentParser):
                                       information doesn't match the current reproducibility information.
         :param skip_unsettable: When True, skips attributes that cannot be set in the Tap object,
                                 e.g. properties without setters.
+        :param repo_path: Path to the git repo to examine for reproducibility info.
+                          If None, uses the git repo of the Python file that is run.
         :return: Returns self.
         """
         with open(path) as f:
@@ -621,7 +669,7 @@ class Tap(ArgumentParser):
         # Remove loaded reproducibility information since it is no longer valid
         saved_reproducibility_data = args_dict.pop('reproducibility', None)
         if check_reproducibility:
-            current_reproducibility_data = self.get_reproducibility_info()
+            current_reproducibility_data = self.get_reproducibility_info(repo_path=repo_path)
             enforce_reproducibility(saved_reproducibility_data, current_reproducibility_data, path)
 
         self.from_dict(args_dict, skip_unsettable=skip_unsettable)
