@@ -24,14 +24,16 @@ from typing import (
     Union,
 )
 from typing_extensions import Literal
-from typing_inspect import get_args, get_origin as typing_inspect_get_origin
+from typing_inspect import get_args as typing_inspect_get_args, get_origin as typing_inspect_get_origin
 
+if sys.version_info >= (3, 10):
+    from types import UnionType
 
 NO_CHANGES_STATUS = """nothing to commit, working tree clean"""
 PRIMITIVES = (str, int, float, bool)
 
 
-def check_output(command: List[str], suppress_stderr: bool = True) -> str:
+def check_output(command: List[str], suppress_stderr: bool = True, **kwargs) -> str:
     """Runs subprocess.check_output and returns the result as a string.
 
     :param command: A list of strings representing the command to run on the command line.
@@ -40,77 +42,79 @@ def check_output(command: List[str], suppress_stderr: bool = True) -> str:
     """
     with open(os.devnull, 'w') as devnull:
         devnull = devnull if suppress_stderr else None
-        output = subprocess.check_output(command, stderr=devnull).decode('utf-8').strip()
+        output = subprocess.check_output(command, stderr=devnull, **kwargs).decode('utf-8').strip()
     return output
 
 
-def has_git() -> bool:
-    """Returns whether git is installed.
+class GitInfo:
+    """Class with helper methods for extracting information about a git repo."""
 
-    :return: True if git is installed, False otherwise.
-    """
-    try:
-        output = check_output(['git', 'rev-parse', '--is-inside-work-tree'])
-        return output == 'true'
-    except (FileNotFoundError, subprocess.CalledProcessError):
-        return False
+    def __init__(self, repo_path: str):
+        self.repo_path = repo_path
 
+    def has_git(self) -> bool:
+        """Returns whether git is installed.
 
-def get_git_root() -> str:
-    """Gets the root directory of the git repo where the command is run.
+        :return: True if git is installed, False otherwise.
+        """
+        try:
+            output = check_output(['git', 'rev-parse', '--is-inside-work-tree'], cwd=self.repo_path)
+            return output == 'true'
+        except (FileNotFoundError, subprocess.CalledProcessError):
+            return False
 
-    :return: The root directory of the current git repo.
-    """
-    return check_output(['git', 'rev-parse', '--show-toplevel'])
+    def get_git_root(self) -> str:
+        """Gets the root directory of the git repo where the command is run.
 
+        :return: The root directory of the current git repo.
+        """
+        return check_output(['git', 'rev-parse', '--show-toplevel'], cwd=self.repo_path)
 
-def get_git_url(commit_hash: bool = True) -> str:
-    """Gets the https url of the git repo where the command is run.
+    def get_git_url(self, commit_hash: bool = True) -> str:
+        """Gets the https url of the git repo where the command is run.
 
-    :param commit_hash: If True, the url links to the latest local git commit hash.
-    If False, the url links to the general git url.
-    :return: The https url of the current git repo.
-    """
-    # Get git url (either https or ssh)
-    try:
-        url = check_output(['git', 'remote', 'get-url', 'origin'])
-    except subprocess.CalledProcessError:
-        # For git versions <2.0
-        url = check_output(['git', 'config', '--get', 'remote.origin.url'])
+        :param commit_hash: If True, the url links to the latest local git commit hash.
+        If False, the url links to the general git url.
+        :return: The https url of the current git repo.
+        """
+        # Get git url (either https or ssh)
+        try:
+            url = check_output(['git', 'remote', 'get-url', 'origin'], cwd=self.repo_path)
+        except subprocess.CalledProcessError:
+            # For git versions <2.0
+            url = check_output(['git', 'config', '--get', 'remote.origin.url'], cwd=self.repo_path)
 
-    # Remove .git at end
-    url = url[:-len('.git')]
+        # Remove .git at end
+        url = url[:-len('.git')]
 
-    # Convert ssh url to https url
-    m = re.search('git@(.+):', url)
-    if m is not None:
-        domain = m.group(1)
-        path = url[m.span()[1]:]
-        url = f'https://{domain}/{path}'
+        # Convert ssh url to https url
+        m = re.search('git@(.+):', url)
+        if m is not None:
+            domain = m.group(1)
+            path = url[m.span()[1]:]
+            url = f'https://{domain}/{path}'
 
-    if commit_hash:
-        # Add tree and hash of current commit
-        url = f'{url}/tree/{get_git_hash()}'
+        if commit_hash:
+            # Add tree and hash of current commit
+            url = f'{url}/tree/{self.get_git_hash()}'
 
-    return url
+        return url
 
+    def get_git_hash(self) -> str:
+        """Gets the git hash of HEAD of the git repo where the command is run.
 
-def get_git_hash() -> str:
-    """Gets the git hash of HEAD of the git repo where the command is run.
+        :return: The git hash of HEAD of the current git repo.
+        """
+        return check_output(['git', 'rev-parse', 'HEAD'], cwd=self.repo_path)
 
-    :return: The git hash of HEAD of the current git repo.
-    """
-    return check_output(['git', 'rev-parse', 'HEAD'])
+    def has_uncommitted_changes(self) -> bool:
+        """Returns whether there are uncommitted changes in the git repo where the command is run.
 
+        :return: True if there are uncommitted changes in the current git repo, False otherwise.
+        """
+        status = check_output(['git', 'status'], cwd=self.repo_path)
 
-def has_uncommitted_changes() -> bool:
-    """Returns whether there are uncommitted changes in the git repo where the command is run.
-
-    :return: True if there are uncommitted changes in the current git repo, False otherwise.
-    """
-    status = check_output(['git', 'status'])
-
-    return not status.endswith(NO_CHANGES_STATUS)
+        return not status.endswith(NO_CHANGES_STATUS)
 
 
 def type_to_str(type_annotation: Union[type, Any]) -> str:
@@ -253,7 +257,7 @@ def get_literals(literal: Literal, variable: str) -> Tuple[Callable[[str], Any],
     literals = list(get_args(literal))
 
     if not all(isinstance(literal, PRIMITIVES) for literal in literals):
-        raise ValueError(
+        raise ArgumentTypeError(
             f'The type for variable "{variable}" contains a literal'
             f'of a non-primitive type e.g. (str, int, float, bool).\n'
             f'Currently only primitive-typed literals are supported.'
@@ -262,7 +266,7 @@ def get_literals(literal: Literal, variable: str) -> Tuple[Callable[[str], Any],
     str_to_literal = {str(literal): literal for literal in literals}
 
     if len(literals) != len(str_to_literal):
-        raise ValueError('All literals must have unique string representations')
+        raise ArgumentTypeError('All literals must have unique string representations')
 
     def var_type(arg: str) -> Any:
         return str_to_literal.get(arg, arg)
@@ -401,7 +405,7 @@ def as_python_object(dct: Any) -> Any:
             return UnpicklableObject()
 
         else:
-            raise ValueError(f'Special type "{_type}" not supported for JSON loading.')
+            raise ArgumentTypeError(f'Special type "{_type}" not supported for JSON loading.')
 
     return dct
 
@@ -469,7 +473,7 @@ def enforce_reproducibility(saved_reproducibility_data: Optional[Dict[str, str]]
                          f'in current args.')
 
 
-# TODO: remove this once typing_inspect.get_origin is fixed for Python 3.8 and 3.9
+# TODO: remove this once typing_inspect.get_origin is fixed for Python 3.8, 3.9, and 3.10
 # https://github.com/ilevkivskyi/typing_inspect/issues/64
 # https://github.com/ilevkivskyi/typing_inspect/issues/65
 def get_origin(tp: Any) -> Any:
@@ -479,4 +483,16 @@ def get_origin(tp: Any) -> Any:
     if origin is None:
         origin = tp
 
+    if sys.version_info >= (3, 10) and isinstance(origin, UnionType):
+        origin = UnionType
+
     return origin
+
+
+# TODO: remove this once typing_insepct.get_args is fixed for Python 3.10 union types
+def get_args(tp: Any) -> Tuple[type, ...]:
+    """Same as typing_inspect.get_args but fixes Python 3.10 union types."""
+    if sys.version_info >= (3, 10) and isinstance(tp, UnionType):
+        return tp.__args__
+
+    return typing_inspect_get_args(tp)
